import subprocess
import logging


logger = logging.getLogger('netd')


def check_output(command, *args, **kwargs):
    '''
    Basically a facade over the check_output
    command that we can do logging with.
    '''
    messages = kwargs.pop('messages', None)
    verbose = kwargs.pop('verbose', False)
    fmt_command = command
    if isinstance(fmt_command, (list, tuple)):
        fmt_command = ' '.join(fmt_command)
    try:
        result = subprocess.check_output(command, *args, **kwargs)
        # success means return code 0
        logger.debug('Ran command: {}, Return code: {}'.format(
            fmt_command, 0))

        if verbose and messages is not None:
            if "output" not in messages:
                messages["output"] = ''
            if result:
                messages["output"] += "{0}\n---------\n{1}\n\n".format(
                    fmt_command, result)
            else:
                messages["output"] += "Ran {0}.\n\n".format(fmt_command)

        return result
    except subprocess.CalledProcessError as exc:
        logger.debug('Ran command: {}, Return code: {}, Output: {}'.format(
            fmt_command, exc.returncode, exc.output))
        if verbose and messages:
            if "output" not in messages:
                messages["output"] = ''
            if exc.output:
                messages["output"] += "{0}\n---------\n{1}\n\n".format(
                    fmt_command, exc.output)
            else:
                messages["output"] += "Ran {0}. Exited {1}\n\n".format(
                    fmt_command, exc.returncode)

        # just bubble, we're here for the logging
        raise


def safe_unidecode(val):
    '''
    Convert a string in a safe way to unicode
    >>> safe_unidecode(b'foobar')
    'foobar'
    :param val: str to convert
    '''
    if isinstance(val, unicode):
        # already decoded
        return val

    for codec in ('utf-8', 'windows-1252', 'latin-1'):
        try:
            return val.decode(codec)
        except UnicodeDecodeError:
            pass
    return val.decode('utf-8', errors='replace')
